@@ -2,7 +2,7 @@
 # Package       HiPi::BCM2835::Pin
 # Description:  Pin
 # Created       Wed Feb 20 04:37:38 2013
-# SVN Id        $Id: Pin.pm 1076 2013-03-13 08:55:10Z Mark Dootson $
+# SVN Id        $Id: Pin.pm 1753 2013-04-25 15:32:30Z Mark Dootson $
 # Copyright:    Copyright (c) 2013 Mark Dootson
 # Licence:      This work is free software; you can redistribute it and/or modify it 
 #               under the terms of the GNU General Public License as published by the 
@@ -21,7 +21,7 @@ use parent qw( HiPi::Pin );
 use Carp;
 use HiPi::Constant qw( :raspberry );
 
-our $VERSION = '0.22';
+our $VERSION ='0.32';
 
 sub _open {
     my ($class, %params) = @_;
@@ -87,5 +87,9 @@ sub _do_setinterrupt {
     HiPi::BCM2835::bcm2835_gpio_set_eds( $_[0]->pinid );
 }
 
+sub _do_setpud {
+    HiPi::BCM2835::bcm2835_gpio_set_pud($_[0]->pinid, $_[1]);
+}
+
 
 1;
