@@ -2,7 +2,7 @@
 # Package       HiPi::Device::GPIO::Pin
 # Description:  Pin
 # Created       Wed Feb 20 04:37:38 2013
-# SVN Id        $Id: Pin.pm 1733 2013-04-23 22:13:29Z Mark Dootson $
+# SVN Id        $Id: Pin.pm 1754 2013-04-27 00:01:50Z Mark Dootson $
 # Copyright:    Copyright (c) 2013 Mark Dootson
 # Licence:      This work is free software; you can redistribute it and/or modify it 
 #               under the terms of the GNU General Public License as published by the 
@@ -21,78 +21,86 @@ use Carp;
 use Fcntl;
 use HiPi::Constant qw( :raspberry );
 use HiPi;
-use IO::File;
 
-our $VERSION = '0.28';
+our $VERSION ='0.32';
 
-__PACKAGE__->create_accessors( qw( valfh pinroot dirfh ) );
+__PACKAGE__->create_accessors( qw( pinroot valfh ) );
 
 sub _open {
     my ($class, %params) = @_;
     defined($params{pinid}) or croak q(pinid not defined in parameters);
     
-    
     my $pinroot = qq(/sys/class/gpio/gpio$params{pinid});
     croak qq(pin $params{pinid} is not exported) if !-d $pinroot;
     
     my $valfile = qq($pinroot/value);
-    my $dirfile = qq($pinroot/direction);
-    $params{valfh} = IO::File->new($valfile, O_RDWR, 0) or croak qq(failed to open $valfile for pin $params{pinid} : $!);
-    $params{dirfh} = IO::File->new($dirfile, O_RDWR, 0) or croak qq(failed to open $dirfile for pin $params{pinid} : $!);
+
+    $params{valfh} = _open_file( $valfile );
     $params{pinroot} = $pinroot;
+    
     my $self = $class->SUPER::_open(%params);
     return $self;
 }
 
+sub _open_file {
+    my $filepath = shift;
+    my $fh;
+    sysopen($fh, $filepath, O_RDWR|O_NONBLOCK) or croak qq(failed to open $filepath : $!);
+    return $fh;
+}
+
+sub _read_fh_bytes {
+    my($fh, $bytes) = @_;
+    my $value;
+    sysseek($fh,0,0);
+    defined( sysread($fh, $value, $bytes) ) or croak(qq(Failed to read from filehandle : $!));
+    chomp $value;
+    return $value;
+}
+
+sub _write_fh {
+    my($fh, $val) = @_;
+    defined( syswrite($fh, $val) ) or croak(qq(Failed to write to filehandle : $!));
+}
+
 sub _do_getvalue {
-    my $self = shift;
-    seek($self->valfh,0,0);
-    my $result;
-    read($self->valfh, $result, 16);
-    chomp($result);
-    return $result;
+    _read_fh_bytes( $_[0]->valfh, 1);
 }
 
 sub _do_setvalue {
-    my ($self, $newval) = @_;
-    write($self->valfh, $newval);
+    my( $self, $newval) = @_;
+    _write_fh($self->valfh, $newval );
+    return $newval;
 }
 
 sub _do_getmode {
     my $self = shift;
-    seek($self->dirfh,0,0);
-    my $result;
-    read($self->dirfh, $result, 16);
-    chomp($result);
+    my $fh = _open_file( $self->pinroot . '/direction' );
+    my $result = _read_fh_bytes( $fh, 16);
+    close($fh);
     return ( $result eq 'out' ) ? RPI_PINMODE_OUTP : RPI_PINMODE_INPT;
 }
 
 sub _do_setmode {
-    my ($self, $newdir) = @_;
-    if( ($newdir == RPI_PINMODE_OUTP) || ($newdir eq 'out') )  {
-        write($self->dirfh, 'out');
+    my ($self, $newmode) = @_;
+    my $fh = _open_file( $self->pinroot . '/direction' );
+    if( ($newmode == RPI_PINMODE_OUTP) || ($newmode eq 'out') )  {
+        _write_fh($fh, 'out');
+        close($fh);
+        return RPI_PINMODE_OUTP;
     } else {
-        write($self->dirfh, 'in');
+        _write_fh($fh, 'in');
+        close($fh);
+        return RPI_PINMODE_INPT;
     }
-    return $newdir;
-}
-
-sub _reset_value_handle {
-    my $self = shift;
-    close($self->valfh);
-    my $filepath = $self->pinroot . '/value';
-    my $newfh = IO::File->new($filepath, O_RDWR, 0) or croak qq(failed to open $filepath : $!);
-    $self->valfh($newfh);
 }
 
 sub _do_getinterrupt {
     my $self = shift;
-    my $edgepath = $self->pinroot . '/edge';
-    my $result = HiPi::qx_sudo(qq(/bin/cat $edgepath));
-    if( $? ) {
-        croak qq(failed reading $edgepath : $!);
-    }
-    chomp( $result );
+    my $fh = _open_file( $self->pinroot . '/edge' );
+    my $result = _read_fh_bytes( $fh, 16);
+    close($fh);
+    
     if($result eq 'rising') {
         return RPI_INT_RISE;
     } elsif($result eq 'falling') {
@@ -106,7 +114,13 @@ sub _do_getinterrupt {
 
 sub _do_setinterrupt {
     my ($self, $newedge) = @_;
+   
     my $stredge = 'none';
+    
+    $newedge = RPI_INT_FALL if $newedge eq 'falling';
+    $newedge = RPI_INT_RISE if $newedge eq 'rising';
+    $newedge = RPI_INT_BOTH if $newedge eq 'both';
+    
     given( $newedge ) {
         when( [ RPI_INT_AFALL, RPI_INT_FALL, RPI_INT_LOW, 'falling'  ] ) {
             $stredge = 'falling';
@@ -122,32 +136,53 @@ sub _do_setinterrupt {
         }
     }
     
-    my $edgepath = $self->pinroot . '/edge';
-    HiPi::system_sudo_shell(qq(/bin/echo $stredge > $edgepath)) and croak qq( failed to write to $edgepath : $!);
+    my $fh = _open_file( $self->pinroot . '/edge' );
+    _write_fh( $fh, $stredge);
+    close($fh);
+    return $newedge;
+}
+
+sub _do_setpud {
+    my($self, $pudval) = @_;
+    my $pudchars = 'error';
+    given( $pudval ) {
+        when([ RPI_PUD_OFF ]) {
+            $pudchars = 'clear';
+        }
+        when([ RPI_PUD_UP ]) {
+            $pudchars = 'up';
+        }
+        when([ RPI_PUD_DOWN ]) {
+            $pudchars = 'down';
+        }
+        default {
+            croak(qq(Incorrect PUD setting $pudval));
+        }
+    }
+    my $pinid = $self->pinid;
+    HiPi::system_sudo(qq(hipi-pud $pinid $pudchars)) and croak qq(failed to set pud restistor for $pinid : $!);
+    return 1;
 }
 
 
-sub active_low {
+sub _do_activelow {
     my($self, $newval) = @_;
     
-    my $filepath = $self->pinroot . '/active_low';
-    
+    my $fh = _open_file( $self->pinroot . '/active_low' );
+    my $result;
     if(defined($newval)) {
-        HiPi::system_sudo_shell(qq(/bin/echo $newval > $filepath)) and croak qq( failed to write to $filepath : $!);
+        _write_fh( $fh, $newval);
+        $result = $newval;
+    } else {
+        $result = _read_fh_bytes( $fh, 1);
     }
-    
-    my $value = HiPi::qx_sudo(qq(/bin/cat $filepath));
-    if($?) {
-        croak qq(failed to read $filepath : $!);
-    }
-    chomp($value);
-    return $value;
+    close($fh);
+    return $result;
 } 
 
 sub DESTROY {
     my $self = shift;
     close($self->valfh);
-    close($self->dirfh);
 }
 
 1;
